function [cwssim, cssim_map] = cw_ssim(img1, img2, level, order, K)

%Decompose images using complex steerable pyramid
%Code from buildSCFpyr documentation
[pyramid1, indicies1] = buildSCFpyr(img1, level, order-1);
[pyramid2, indicies2] = buildSCFpyr(img2, level, order-1);

%Create normalized window of size 7
winsize = 7;
window = ones(7);
window = window./sum(sum(window));

% The CW-SSIM index map is combined into a scalar similarity measure using a
% weighted summation.The weighting function is obtained using a Gaussian 
% profile with a standard deviation equaling a quarter of the image size at 
% finest level of pyramid decomposition.
% Reference: Page 6 of samtap_tip_nov09.pdf
s = indicies1((level-1)*order+2, :)
w = fspecial('gaussian', s-winsize+1, s(1)/4);

% process each sub band in pyramid
for i=1:order
   bandind = i+(level-1)*order+1;
   band1 = pyrBand(pyramid1, indicies1, bandind);
   band2 = pyrBand(pyramid2, indicies2, bandind);
   band1 = band1(1:end, 1:end);
   band2 = band2(1:end, 1:end);
   
   % implement part 1 of equation (9)
   % Reference: Page 4 of samtap_tip_nov09.pdf
   corr = band1.*conj(band2);
   varr = abs(band1).^2 + abs(band2).^2;
   
   % using matlab's filter2 command to extract elements within window
   corr_band = filter2(window, corr, 'valid');
   varr_band = filter2(window, varr, 'valid');

   % implement part 2 of equation (9)
   % Reference: Page 4 of samtap_tip_nov09.pdf
   cssim_map = ...
   (2*abs(corr_band) + K)./(varr_band + K);

   band_cssim(i) = sum(sum(cssim_map.*w));% Apply weighting function w
   
end

cwssim = mean(band_cssim);
