function [quality] = miqe_video_csfd2(imorg,imdist,vidFileName, qpVal, motFile,distMotFile,motionType,csfType,frameRate,reflev, shift)

%Input : (1) imorg: The reference image
%        (2) imdist: The distorted image (order is important)
%        (3) motFile: The default motion file prefix for different direction and subbands
%        (4) motionType: 0 : Block Motion (no 4x4 operation is required)
%                        1 : Watson-Ahumada optical flow based motion (4x4 operation is required)
%        (5) csfType:    0 : Daly SpatioTemporal Contrast Sensitivity Function
%                        1 : Kelly SpatioTemporal Contrast Sensitivity Function
%                        2 : Saliency based SpatioTemporal Sensitivity Function
%Output: (1) VIF te visual information fidelity measure between the two images

%Default Usage:
%   Given 2 test images img1 and img2, whose dynamic range is 0-255
%
%   quality = miqe_video(imorg,imdist,'motion');
%
%Advanced Usage:
%   Users may want to modify the parameters in the code. 
%   (1) Modify sigma_nsq to find tune for your image dataset.
%   (2) MxM is the block size that denotes the size of a vector used in the
%   GSM model.
%   (3) subbands included in the computation
%========================================================================

M=4; %non-overlapping block size is 4 (3 before)
%reflev = 4; %5
subbands=1:3*reflev+1;
sigma_nsq=0.4;

% 9/7 Discrete wavelet decomposition. 

%1) Pixel value to physical luminance
% display device black-level offset (sRGB)
b = 0;
% display device pixel-value-to-voltage gain (sRGB)
k = 0.02874;
% display device gamma (sRGB)
g = 2.2;
% initialize the pixel-value-to-luminance lookup table
pixel_vals = -255:255;
pix2lum_table =abs((b + k*pixel_vals)).^g;
for row = 1 : size(imorg,1)     
    imorg(row,:) = pix2lum_table(imorg(row,:)+256);
end
for row = 1 : size(imdist,1)     
    imdist(row,:) = pix2lum_table(imdist(row,:)+256);
end

%Optional : Luminance masking
%

% [imorg, imdist] = getCsf(imorg,imdist);
ref = dwt2d_norm(imorg,reflev,'97');
[rowSz,colSz] = size(imorg);
rowLevSz = rowSz / (2^(reflev));
colLevSz = colSz / (2^(reflev)); 
levSzPrv = 1;
ind = 1;
% Reference Wavelet Decomposition
for kk =1:reflev
    if (kk == 1)
        orgArr{ind} = ref(1:rowLevSz,1:colLevSz);
        ind = ind +1;
    end
    orgArr{ind}     = ref(rowLevSz+1:2*rowLevSz,1:colLevSz);
    orgArr{ind + 1} = ref(rowLevSz+1:2*rowLevSz,colLevSz+1:2*colLevSz);
    orgArr{ind + 2} = ref(1:rowLevSz,colLevSz+1:2*colLevSz);
    ind = ind + 3;
    rowLevSz = 2*rowLevSz;
    colLevSz = 2*colLevSz;
end

% Distance over height and test  
if(size(imdist,1) == size(imorg,1))
    test = dwt2d_norm(imdist,reflev,'97');
    doh = 2;% * (720/rowSz);% 512 -> 720 
    subbands_dist = subbands;
    distlev = reflev;
else
    test = dwt2d_norm(imdist,reflev-1,'97');
    doh =4;% * (720/rowSz);% 512 -> 720
    subbands_dist = subbands(1:end-3);
    distlev = reflev - 1;
end

rowLevSz = rowSz / (2^(reflev));
colLevSz = colSz / (2^(reflev)); 
ind = 1;


for kk =1:reflev
    if (kk == 1)
        distArr{ind} = test(1:rowLevSz,1:colLevSz);
        ind = ind +1;
    end
    if( (size(imdist,1) == size(imorg,1)) || (kk < reflev))
        distArr{ind}     = test(rowLevSz+1:2*rowLevSz,1:colLevSz);
        distArr{ind + 1} = test(rowLevSz+1:2*rowLevSz,colLevSz+1:2*colLevSz);
        distArr{ind + 2} = test(1:rowLevSz,colLevSz+1:2*colLevSz);
    else
        distArr{ind}     = zeros(rowLevSz,colLevSz);
        distArr{ind + 1} = zeros(rowLevSz,colLevSz);
        distArr{ind + 2} = zeros(rowLevSz,colLevSz);        
    end
    ind = ind + 3;
    rowLevSz = 2*rowLevSz;
    colLevSz = 2*colLevSz;
end


% Contrast Sensitivity Function
% Computing Spatial Frequency
a = 0.31; 
b= 0.69; 
c = 0.29;
fs = rowSz/2;
lambda = 1:3;
la_vec = [];
for ii = reflev : -1 : 1
    if(ii == reflev)
        la_vec = [la_vec ii*ones(1,4)];%lambda vector for each subband
    else
        la_vec = [la_vec ii*ones(1,3)];
    end
end
subband = 1:3*reflev+1;
% F_lambda1 = (pi * fs * doh)./(180 * 2.^(la_vec(subband)));

or_ind = [2:3:3*reflev+1 4:3:3*reflev+1];
or =-ones(1,3*reflev+1);
or(or_ind) = 1;

F_lambda = (pi * fs * doh)./(180 * 2.^(la_vec(subband)));
%w is the spatial frequency value
w =  F_lambda./(0.85 + 0.15 * or);
% Computing Temporal Frequency
% - Extracting motion information to motionCell for each subband
% or subband level. Format of the motion block is 4x4 block for each non
% overlapping block. Save the motion information before
fun = @(block_struct) mean(mean(block_struct.data));
% Original video motion
if(motionType == 0)
    %disp('murat');
elseif(motionType == 1)
%     if(reflev <=3)
        for ii = 1:3*reflev+1
            loadMotion = ['load ' motFile 'x' num2str(ii)];
            eval(loadMotion);
            loadMotion = ['load ' motFile 'y' num2str(ii)];
            eval(loadMotion);
            motionX{ii} = mX;
            motionY{ii} = mY;    
            motion{ii} = sqrt(motionX{ii}.*motionX{ii} + motionY{ii}.*motionY{ii});
        end
        for ii = 1:length(subbands_dist)
            loadMotion = ['load ' distMotFile 'x' num2str(ii)];
            eval(loadMotion);
            loadMotion = ['load ' distMotFile 'y' num2str(ii)];
            eval(loadMotion);
            distMotionX{ii} = mX;
            distMotionY{ii} = mY;    
            distMotion{ii} = sqrt(distMotionX{ii}.*distMotionX{ii} + distMotionY{ii}.*distMotionY{ii});
        end
%     elseif(reflev ==4)
%         for ii = 1:10
%             loadMotion = ['load ' motFile 'x' num2str(ii)];
%             eval(loadMotion);
%             loadMotion = ['load ' motFile 'y' num2str(ii)];
%             eval(loadMotion);
%             motionX{ii} = mX;
%             motionY{ii} = mY;    
%             motion{ii+3} = sqrt(motionX{ii}.*motionX{ii} + motionY{ii}.*motionY{ii});
%         end
%         for ii = 1:length(subbands_dist)-3
%             loadMotion = ['load ' distMotFile 'x' num2str(ii)];
%             eval(loadMotion);
%             loadMotion = ['load ' distMotFile 'y' num2str(ii)];
%             eval(loadMotion);
%             distMotionX{ii} = mX;
%             distMotionY{ii} = mY;    
%             distMotion{ii+3} = sqrt(distMotionX{ii}.*distMotionX{ii} + distMotionY{ii}.*distMotionY{ii});
%         end
%             motion{1} = blockproc(motion{4},[2 2],fun);
%             motion{2} = motion{1};
%             motion{3} = motion{1};
%             motion{4} = motion{1};  
%             distMotion{1} = blockproc(distMotion{4},[2 2],fun);
%             distMotion{2} = distMotion{1};
%             distMotion{3} = distMotion{1};
%             distMotion{4} = distMotion{1};            
%     end
else
% Reference video motion
    for ii = 1:reflev
        mX = dlmread([vidFileName '\QP1\WA\' motFile '_x' num2str(ii-1) '.txt'], ' ');
        mY = dlmread([vidFileName '\QP1\WA\' motFile '_y' num2str(ii-1) '.txt'], ' ');
        mY = (-mY(:,end:-1:1))';
        mX = (mX(:,end:-1:1))';
        mX = blockproc(mX,[8 8],fun);
        mY = blockproc(mY,[8 8],fun);
        motion{ii} = (sqrt(mX.*mX + mY.*mY));%/((rowSz*colSz)^(0.5)*(4^(ii-1))*3);
    end
% Distorted video motion    
    for ii = 1:distlev
        mX = dlmread([vidFileName '\QP1' '\WA\' distMotFile '_x' num2str(ii-1) '.txt'], ' '); %num2str(qpVal)
        mY = dlmread([vidFileName '\QP1' '\WA\' distMotFile '_y' num2str(ii-1) '.txt'], ' '); %num2str(qpVal) 
        mY = (-mY(:,end:-1:1))';
        mX = (mX(:,end:-1:1))';
        mX = blockproc(mX,[8 8],fun);
        mY = blockproc(mY,[8 8],fun);
        distMotion{ii} = (sqrt(mX.*mX + mY.*mY));%/((rowSz*colSz)^(0.5)*4^(ii-1)*2^(distlev-reflev)*3);%^0.5
    end
end

% H = (a+b*w).* exp(-c*w);  % CSF value for each subband kang li
% H = 59.8*w.^(-0.1087*log(w)-0.1258);
% H = 1;
% end

% Decoupling

% calculate the parameters of the distortion channel
[g_all,vv_all]=vifsub_est_m(orgArr,distArr,subbands_dist,M);

% calculate the parameters of the reference image
[ssarr, larr, cuarr]=refparams_vecgsm(orgArr,subbands,M);

% reorder subbands. This is needed since the outputs of the above functions
% are not in the same order
vvtemp=cell(1,max(subbands_dist));
ggtemp=vvtemp;
for(kk=1:length(subbands_dist))
    vvtemp{subbands_dist(kk)}=vv_all{kk};
    ggtemp{subbands_dist(kk)}=g_all{kk};
end

% Mutual Information

% compute reference and distorted image information from each subband
for i=1:length(subbands)%2
    sub=subbands(i);
    if(sub <= length(subbands_dist))    
        g=ggtemp{sub};
        vv=vvtemp{sub};
    end
    ss=ssarr{sub};
    lambda = larr(sub,:);
    cu=cuarr{sub};

    % how many eigenvalues to sum over. default is all.
    neigvals=length(lambda);
    
    % compute the size of the window used in the distortion channel estimation, 
    % and use it to calculate the offset from subband borders
    % we do this to avoid all coefficients that may suffer from boundary
    % effects
    lev=ceil((sub-1)/6);
    winsize=2^lev+1; offset=(winsize-1)/2;
    offset=ceil(offset/M);
    
    %
    if(motionType == 0)
        %disp('murat');
    elseif(motionType == 1)
        motInd = i;
        distMotInd = i; 
    else%13,12,11-0, 10,9,8-1, 7,6,5-2, 4,3,2-3 dM : 10,9,8-0, 7,6,5-1, 4,3,2-2   
        motInd = min(5 - floor((i+1)/3),4);
        if(size(imdist,1) == size(imorg,1))
            distMotInd = motInd;
        else
            distMotInd = motInd - 1;
        end
    end
    if(motionType > 0)    
        csfVal = csfFunc2(w(i),motion{motInd},csfType,frameRate,shift);
    else
        csfVal = csfFunc2(w(i),zeros(size(ss)),csfType,frameRate,shift);        
    end
    csfVal = csfVal(offset+1:end-offset,offset+1:end-offset);
    % select only valid portion of the output.
    if(sub <= length(subbands_dist))
        g=g(offset+1:end-offset,offset+1:end-offset);
        vv=vv(offset+1:end-offset,offset+1:end-offset);
        if(motionType > 0)    
            distCsfVal = csfFunc2(w(i),distMotion{distMotInd},csfType,frameRate,shift);
        else
            distCsfVal = csfFunc2(w(i),zeros(size(ss)),csfType,frameRate,shift);
        end
        distCsfVal = distCsfVal(offset+1:end-offset,offset+1:end-offset);
    end
    ss=ss(offset+1:end-offset,offset+1:end-offset);
    
    
    
    %VIF
    sz_g = size(ss);
    temp1=0; temp2=0;
    for j=1:length(lambda)
        if(sub <= length(subbands_dist))
            temp1=temp1+sum(sum(distCsfVal(1:sz_g(1),1:sz_g(2)).*(log2(1+g.*g.*ss.*lambda(j)./(vv+sigma_nsq))))); % distorted image information for the i'th subband
        end
        temp2=temp2+sum(sum(csfVal(1:sz_g(1),1:sz_g(2)).*(log2(1+ss.*lambda(j)./(sigma_nsq))))); % reference image information
    end
    num(i)=temp1;
    den(i)=temp2;
end
rouse = 1:3*reflev+1;
sW = 2.^((reflev - ceil((rouse -1)./3))*2);
sWH = sW;%.*(H);
% sWH(end-2:end) = sWH(end-2:end) * 0.75; 
% compuate VIF
% vif=sum(sWH(2:end).*den(2:end));
a1 = sW(2:end).*num(2:end);
quality=sum(sW(1:end).*num(1:end))./sum(sW(1:end).*den(1:end));%2
%vif=sum(num)./sum(den);
