function [vif]=vifvec_li(imorg,imdist);


% -----------COPYRIGHT NOTICE STARTS WITH THIS LINE------------
% Copyright (c) 2005 The University of Texas at Austin
% All rights reserved.
% 
% Permission is hereby granted, without written agreement and without license or royalty fees, to use, copy, 
% modify, and distribute this code (the source files) and its documentation for
% any purpose, provided that the copyright notice in its entirety appear in all copies of this code, and the 
% original source of this code, Laboratory for Image and Video Engineering (LIVE, http://live.ece.utexas.edu)
% at the University of Texas at Austin (UT Austin, 
% http://www.utexas.edu), is acknowledged in any publication that reports research using this code. The research
% is to be cited in the bibliography as:
% 
% H. R. Sheikh and A. C. Bovik, "Image Information and Visual Quality", IEEE Transactions on 
% Image Processing, (to appear).
% 
% IN NO EVENT SHALL THE UNIVERSITY OF TEXAS AT AUSTIN BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
% OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS DATABASE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF TEXAS
% AT AUSTIN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% THE UNIVERSITY OF TEXAS AT AUSTIN SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE DATABASE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
% AND THE UNIVERSITY OF TEXAS AT AUSTIN HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
% 
% -----------COPYRIGHT NOTICE ENDS WITH THIS LINE------------
%
%This is an implementation of the algorithm for calculating the
%Visual Information Fidelity (VIF) measure (may also be known as the Sheikh
%-Bovik Index) between two images. Please refer
%to the following paper:
%
%H. R. Sheikh and A. C. Bovik "Image Information and Visual Quality"
%IEEE Transactios on Image Processing, in publication, May 2005.
%Download manuscript draft from http://live.ece.utexas.edu in the
%Publications link
%
%This implementation is slightly differnet from the one used to report
%results in the paper above. The modification have to do with using more
%subands than those used in the paper, better handling of image boundaries,
%and a window that automatically resizes itself based on the scale.
%
%Report bugfixes and comments to hamid.sheikh@ieee.org
%
%----------------------------------------------------------------------
% Prerequisites: The Steerable Pyramid toolbox. Available at
% http://www.cns.nyu.edu/~lcv/software.html
%
%Input : (1) img1: The reference image
%        (2) img2: The distorted image (order is important)
%
%Output: (1) VIF te visual information fidelity measure between the two images

%Default Usage:
%   Given 2 test images img1 and img2, whose dynamic range is 0-255
%
%   vif = vifvec(img1, img2);
%
%Advanced Usage:
%   Users may want to modify the parameters in the code. 
%   (1) Modify sigma_nsq to find tune for your image dataset.
%   (2) MxM is the block size that denotes the size of a vector used in the
%   GSM model.
%   (3) subbands included in the computation
%========================================================================

M=3;
reflev = 5;
subbands=1:3*reflev+1;
sigma_nsq=0.4;

% Do wavelet decomposition. This requires the Steerable Pyramid. You can
% use your own wavelet as long as the cell arrays org and dist contain
% corresponding subbands from the reference and the distorted images
% respectively.

%[pyr,pind] = buildSpyr(imorg, 1, 'sp1Filters', 'reflect1'); % compute transform
%org=ind2wtree(pyr,pind); % convert to cell array
%gamma correction
% display device black-level offset (sRGB)
b = 0;
% display device pixel-value-to-voltage gain (sRGB)
k = 0.02874;
% display device gamma (sRGB)
g = 2.2;
% initialize the pixel-value-to-luminance lookup table
pixel_vals = -255:255;
pix2lum_table =abs((b + k*pixel_vals)).^g;
for row = 1 : size(imorg,1)     
    imorg(row,:) = pix2lum_table(imorg(row,:)+256);
end
for row = 1 : size(imdist,1)     
    imdist(row,:) = pix2lum_table(imdist(row,:)+256);
end

% [imorg, imdist] = getCsf(imorg,imdist);
% [imorg, imdist] = getCsf(imorg,imdist);

ref = dwt2d(imorg,reflev,'97');
% imdist = dwt2d(imorg,1,'97');
% rec = idwt2d(ref,reflev,'97');
[rowSz,colSz] = size(imorg);
rowLevSz = rowSz / (2^(reflev));
colLevSz = colSz / (2^(reflev)); 
levSzPrv = 1;
ind = 1;



a = 0.31; 
b= 0.69; 
c = 0.29;
fs = 256;

if(size(imdist,1) == size(imorg,1))
    test = double(dwt2d(imdist,reflev,'97'));
    doh = 3;
    N = rowSz * colSz;
    subbands_dist = subbands;
else
    test = double(dwt2d(imdist,reflev-1,'97'));
    doh = 6;
    N = rowSz * colSz / 4;
    subbands_dist = subbands(1:end-3);
end




% doh = 4;
lambda = 1:3;
la_vec = [];
for ii = reflev : -1 : 1
    if(ii == reflev)
        la_vec = [la_vec ii*ones(1,4)];%lambda vector for each subband
    else
        la_vec = [la_vec ii*ones(1,3)];
    end
end
subband = 1:3*reflev+1;
% F_lambda1 = (pi * fs * doh)./(180 * 2.^(la_vec(subband)));

or_ind = [2:3:3*reflev+1 4:3:3*reflev+1];
or =-ones(1,3*reflev+1);
or(or_ind) = 1;
% w =  F_lambda ./(0.85 + 0.15 * or);
% H = (a+b*w).* exp(-c*w);  % CSF value for each subband
% doh = 8;
F_lambda = (pi * fs * doh)./(180 * 2.^(la_vec(subband)));
w =  F_lambda ./(0.85 + 0.15 * or);
H = (a+b*w).* exp(-c*w);  % CSF value for each subband kang li
% H = 59.8*w.^(-0.1087*log(w)-0.1258).*H;
% H = 1;
% end





for kk =1:reflev
    if (kk == 1)
        orgArr{ind} = ref(1:rowLevSz,1:colLevSz)*H(ind);
        ind = ind +1;
    end
    orgArr{ind}     = ref(rowLevSz+1:2*rowLevSz,1:colLevSz)*H(ind);
    orgArr{ind + 1} = ref(rowLevSz+1:2*rowLevSz,colLevSz+1:2*colLevSz)*H(ind);
    orgArr{ind + 2} = ref(1:rowLevSz,colLevSz+1:2*colLevSz)*H(ind);
    ind = ind + 3;
    rowLevSz = 2*rowLevSz;
    colLevSz = 2*colLevSz;
end



rowLevSz = rowSz / (2^(reflev));
colLevSz = colSz / (2^(reflev)); 
ind = 1;


for kk =1:reflev
    if (kk == 1)
        distArr{ind} = test(1:rowLevSz,1:colLevSz)*H(ind);
        ind = ind +1;
    end
    if( (size(imdist,1) == size(imorg,1)) || (kk < reflev))
        distArr{ind}     = test(rowLevSz+1:2*rowLevSz,1:colLevSz)*H(ind);
        distArr{ind + 1} = test(rowLevSz+1:2*rowLevSz,colLevSz+1:2*colLevSz)*H(ind);
        distArr{ind + 2} = test(1:rowLevSz,colLevSz+1:2*colLevSz)*H(ind);
    else
        distArr{ind}     = zeros(rowLevSz,colLevSz);
        distArr{ind + 1} = zeros(rowLevSz,colLevSz);
        distArr{ind + 2} = zeros(rowLevSz,colLevSz);        
    end
    ind = ind + 3;
    rowLevSz = 2*rowLevSz;
    colLevSz = 2*colLevSz;
end



% calculate the parameters of the distortion channel
[g_all,vv_all]=vifsub_est_m(orgArr,distArr,subbands_dist,M);

% calculate the parameters of the reference image
[ssarr, larr, cuarr]=refparams_vecgsm(orgArr,subbands,M);

% reorder subbands. This is needed since the outputs of the above functions
% are not in the same order
vvtemp=cell(1,max(subbands_dist));
ggtemp=vvtemp;
for(kk=1:length(subbands_dist))
    vvtemp{subbands_dist(kk)}=vv_all{kk};
    ggtemp{subbands_dist(kk)}=g_all{kk};
end


% compute reference and distorted image information from each subband
for i=2:length(subbands)
    sub=subbands(i);
    if(sub <= length(subbands_dist))    
        g=ggtemp{sub};
        vv=vvtemp{sub};
    end
    ss=ssarr{sub};
    lambda = larr(sub,:);
    cu=cuarr{sub};

    % how many eigenvalues to sum over. default is all.
    neigvals=length(lambda);
    
    % compute the size of the window used in the distortion channel estimation, and use it to calculate the offset from subband borders
    % we do this to avoid all coefficients that may suffer from boundary
    % effects
    lev=ceil((sub-1)/6);
    winsize=2^lev+1; offset=(winsize-1)/2;
    offset=ceil(offset/M);
    
    
    % select only valid portion of the output.
    if(sub <= length(subbands_dist))
        g=g(offset+1:end-offset,offset+1:end-offset);
        vv=vv(offset+1:end-offset,offset+1:end-offset);
    end
    ss=ss(offset+1:end-offset,offset+1:end-offset);
    
    
    %VIF
    temp1=0; temp2=0;
    for j=1:length(lambda)
        if(sub <= length(subbands_dist))
            temp1=temp1+sum(sum((log2(1+g.*g.*ss.*lambda(j)./(vv+sigma_nsq))))); % distorted image information for the i'th subband
        end
        temp2=temp2+sum(sum((log2(1+ss.*lambda(j)./(sigma_nsq))))); % reference image information
    end
    num(i)=temp1;
    den(i)=temp2;
    
end
rouse = 1:3*reflev+1;
sW = 2.^((reflev - ceil((rouse -1)./3))*2);
sWH = sW;%.*(H);
% sWH(end-2:end) = sWH(end-2:end) * 0.75; 
% compuate VIF
% vif=sum(sWH(2:end).*den(2:end));
a1 = sWH(2:end).*num(2:end);
vif=sum(sWH(2:end).*num(2:end))./sum(sWH(2:end).*den(2:end));
%vif=sum(num)./sum(den);
