function [videoQuality] = videoQualityEstimation(refv, distv, fileName, totalFrameNum, frameRate, motionStyle, motionFolder, dim)
%Input : (1) refv: The reference video 
%        (2) distv: The reference video
%        (3) fileMane: fileName prefix for motion files
%        (4) motFile: The default motion file prefix for different direction and subbands
%        (5) totalFrameNum: Total number of frames
%        (6) frameRate: The frame rate of the video
%        (7) motionStyle: 0 : Block Motion (no 4x4 operation is required)
%                         1 : Watson-Ahumada optical flow based motion (4x4 operation is required)
%        (8) motionFolder: Name of the folder that keeps motion files
%        (9) dim : A vector consists of 2 variable width = dim(1), height = dim(2)
%Output: (1) videoQuality : The estimated video quality value

%Default Usage:
%   videoQuality = videoQualityEstimation('ParkJoy_420_720p50.avi', 'ParkJoy_420_720p50_ds.avi', 'park1', 500, 50, 2, 'parkFold', [1280 720])

warning off;
figureon = 0;
BlockSize = [4,4];
numOfFrames = 104;
reflev = 4;
QPind = 0;
numFrameGroup = 8;
csfType = 0;
motionEstTypes = {'HB', 'WA'};
quality = zeros(1,totalFrameNum);
numOfParts = floor(totalFrameNum/numOfFrames);
numOfGroup = floor(totalFrameNum/numFrameGroup);
if motionStyle == 1
    folder = [motionFolder '\'];%'hhhhhh\' ;% 'aaaaaa\'
else
    folder = '';
end
distFileName = ['dist_' fileName];
dirName = [fileName '\QP0\' motionEstTypes{motionStyle}];
if(~isdir(dirName))
    mkdir(dirName);
end
waDir = '';%'C:\Users\ademirta\Downloads\WAv2\WA\bin\Debug\';
imDir = 'C:\c#\optic_flow_test_data\soccer\';
wa_exe = 'WA.exe ';
if(~ exist([distv]))
    numfrm = yuv2avi([distv(1:end-5) '.yuv'],[dim(1)/2 dim(2)/2],[distv(1:end-5) '.avi'],'none',frameRate,frameNum);
end
if(~ exist([refv]))
    numfrm = yuv2avi([refv(1:end-5) '.yuv'],[dim(1) dim(2)],[refv(1:end-5) '.avi'],'none',frameRate,frameNum);
end

for part = 0:numOfParts
    if((part+1)*numOfFrames+1 > totalFrameNum)
        finalFrame = totalFrameNum;
    else
        finalFrame = (part+1)*numOfFrames+1;
    end
    cvid = aviread(refv,[part*numOfFrames+1:finalFrame]);
    distvid = aviread(distv,[part*numOfFrames+1:finalFrame]);
    motionVect = [];
    [RowSize,ColSize] = size(rgb2gray(cvid(1).cdata));
    [distRowSize,distColSize] = size(rgb2gray(distvid(1).cdata));
    dirName = [fileName '\QP' num2str(QPind) '\' motionEstTypes{motionStyle}];
    if(~isdir(dirName))
        mkdir(dirName);
    end
    if(~isdir(folder))
        mkdir(folder);
    end
    %   According to the type of motion estimation, the correponding video quality 
    %   estimation implemetation will be either for a group of frames
    %   or for a single frame motion. The number of frames used for each
    %   watson-ahumada motion estimation operation is 8. Therefore we will use
    %   the same motion file for the following 8 frames. Motion sytle 0 is
    %   regular block based motion estimation. Motion style 1 is watson ahumada
    
    startFrame = part*numOfFrames;
    for ind=startFrame+1:startFrame+numOfFrames
        fNum = ind - startFrame;
        if(motionStyle == 1)
            if ind == totalFrameNum
                break;
            end                       
            motionFile = [fileName '_F' num2str(ind)];
    % -------------Motion Estimation Style 1----------------------

           %For the reference video only one time
           motionHBMA4x4(rgb2gray(cvid(fNum).cdata), rgb2gray(cvid(fNum+1).cdata) , BlockSize , [-16 -16] ,[16,16], figureon, reflev, [folder motionFile]);
            if(RowSize == distRowSize)
                distMotionFile = [distFileName '_QP' num2str(QPind) '_F' num2str(ind)];
                motionHBMA4x4(rgb2gray(distvid(fNum).cdata), rgb2gray(distvid(fNum+1).cdata) , BlockSize , [-16 -16] ,[16,16], figureon, reflev, [folder distMotionFile]);
            else
               distMotionFile = [distFileName '_ds_QP' num2str(QPind) 'F' num2str(ind)];                
               motionHBMA4x4(rgb2gray(distvid(fNum).cdata), rgb2gray(distvid(fNum+1).cdata) , BlockSize , [-16 -16] ,[16,16], figureon, reflev-1, [folder distMotionFile]);
            end

%            quality(ind) = miqe_video_csf(imorg,imdist,fileName,QPind,[folder motionFile],[folder distMotionFile], motionStyle,csfType,frameRate,reflev);

        else
            if ind == numOfGroup*numFrameGroup
                break;
            end                    
            if(mod(fNum,8) == 1)
%                 for ind = 1:floor(numOfFrames/numFrameGroup) 
                    motionFile = [fileName '_QP0_F' num2str(ind)];
                    if(RowSize == distRowSize)
                        distMotionFile = [distFileName '_QP' num2str(QPind) '_F' num2str(ind)];
                    else
                        distMotionFile = [distFileName '_ds_QP' num2str(QPind) 'F' num2str(ind)];                
                    end
                    %Write the frames to the files
                    %Calling watson ahumada to estimate motion. Make watson ahumda
                    %parametric such that we can call it from matlab
                    if(exist([fileName '/QP0/WA/' motionFile '_x0.txt'],'file') == 0)
                        for kk = 1:numFrameGroup
                            imwrite( cvid((fNum-1)+kk).cdata, [imDir fileName num2str(kk-1) '.bmp']);    
                        end
                        cmd = [waDir wa_exe num2str(numFrameGroup) ' ' fileName ' 0 ' motionFile];
                        system(cmd);
                    end

                    if(exist([fileName '/QP' num2str(QPind) '/WA/' distMotionFile '_x0.txt'],'file') == 0)
                        for kk = 1:numFrameGroup
                            imwrite( distvid((fNum-1)+kk).cdata, [imDir fileName num2str(kk-1) '.bmp']);       
                        end
                        cmd = [waDir wa_exe num2str(numFrameGroup) ' ' fileName ' ' num2str(QPind) ' ' distMotionFile];
                        system(cmd);
                    end
                    %Use the estimated motion information for the following 8 frames and
                    %estimate the quality using this motion information
            end
        end
    % -------------Quality Estimation----------------------
        imorg = double(rgb2gray(cvid(fNum).cdata));
        imdist = double(rgb2gray(distvid(fNum).cdata));
        quality(ind) = miqe_video_csf_dst(imorg,imdist,fileName,QPind,[folder motionFile],[folder distMotionFile], motionStyle,csfType,frameRate,reflev,1.3);
        disp(['Frame Num: ' num2str(ind)]);
    end
    %fclose(fid);
    
end
videoQuality = mean(quality);
%delete([distv]);

